/*
 *
 *  ALL03
 *
 *  21.05.2003
 *
 *  by Fabio Sturman
 *
 *  fabio.sturman@tiscali.it
 *
 *  89s52.c
 *
 *  main module for 89s52
 *
 */
/* Copyright (c) 2002, 2003 Fabio Sturman (fabio.sturman@tiscali.it)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>
#include "../include/all03.h"
#include "../include/all03ft.h"
#include "../include/keys.h"
#include "../include/sleep.h"
#include "../include/binmask.h"
#include "89s5x.h"

/*- AT89S52 Level 1 --------------------------------------*/
int pin_miso=PIN_MISO_AT89S52;
int pin_mosi=PIN_MOSI_AT89S52;
int pin_sck=PIN_SCK_AT89S52;
int pin_rst=PIN_RST_AT89S52;
int pin_vcc=PIN_VCC_AT89S52;
int pin_clock1=PIN_CLOCK1_AT89S52;
int pin_clock2=PIN_CLOCK2_AT89S52;

/*- AT98S52 Level 2 --------------------------------------*/
char chip_model[64]="AT89S52";
char adapter_model[64]="A40OSC";
unsigned short int max_flash_address=0x2000;
unsigned char flash[0x4000]; /* 32k */
int lock_bit_1=1;
int lock_bit_2=1;
int lock_bit_3=1;
unsigned char signature[64];
char flash_file_name[256]="mb21.bin";

/*- AT89S52 Level 1 --------------------------------------*/
/* write read */


//OKKKKKKKKKK
unsigned char wr_rd_serial(unsigned char byt)
{
  int i;
  unsigned char rd;

  rd=0;
  for(i=0; i<8; i++)
  {
    set_mosi(byt & BIT(7-i) ? 1 : 0);
    set_sck(1);
    if(get_miso()) rd|=BIT(7-i);
    set_sck(0);
  }
  set_mosi(0);
  return(rd);
}

/* put chip in programming mode */
void chip_init(void)
{
  all03_set_vcc(5.0);
  all03_set_gnd_20();
  set_power();
  set_clock();
  set_miso(1);
  set_rst(1);
  set_sck(0);
  wait(10.0);
}

unsigned char cmd_programming_enable(void)
{
  unsigned char t;

  t=wr_rd_serial(CMD_PROGRAMMING_ENABLE_1);
  t=wr_rd_serial(CMD_PROGRAMMING_ENABLE_2);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
  //DEBUG  printf("{%02X}",t);
  return(t);
}

void cmd_chip_erase(void)
{
  unsigned char t;

  t=wr_rd_serial(CMD_CHIP_ERASE_1);
  t=wr_rd_serial(CMD_CHIP_ERASE_2);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
}

unsigned char cmd_read_program_memory(unsigned short int address)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_PROGRAM_MEMORY);
  t=wr_rd_serial(address / 256);
  t=wr_rd_serial(address & 0xff);
  t=wr_rd_serial(0xff);
  return(t);
}

void cmd_write_program_memory(unsigned short int address, unsigned char data)
{
  unsigned char t;

  t=wr_rd_serial(CMD_WRITE_PROGRAM_MEMORY);
  t=wr_rd_serial(address / 256);
  t=wr_rd_serial(address & 0xff);
  t=wr_rd_serial(data);
}

unsigned char cmd_read_lock_bits(void)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_LOCK_BITS);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
  return(t);
}

void cmd_write_lock_bits(int lock_bit_1, int lock_bit_2)
{
  unsigned char t;

  t=wr_rd_serial(CMD_WRITE_LOCK_BITS_1);
  t=wr_rd_serial(CMD_WRITE_LOCK_BITS_2 | (lock_bit_1 ? BIT(1) : 0) | (lock_bit_2 ? BIT(0) : 0) );
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
}

unsigned char cmd_read_signature_bytes(unsigned short int address)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_SIGNATURE_BYTES);
  t=wr_rd_serial(address & M_0011_1111);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
  //DEBUGprintf("{%02X}",t);
  return(t);
}

int wait_rdy_program(unsigned short int address, unsigned char data)
{
  long t, to;

  to=get_one_ms()*2;
  for(t=0; t<to; t++)
    if(data==cmd_read_program_memory(address)) return(0);
  return(1);
}

/*- AT89S52 Level 2 --------------------------------------*/
void select_model_at89s52(void)
{
  strcpy(chip_model,"AT89S52");
  strcpy(adapter_model,"A40OSC");
  max_flash_address =0x2000;
  pin_miso=PIN_MISO_AT89S52;
  pin_mosi=PIN_MOSI_AT89S52;
  pin_sck=PIN_SCK_AT89S52;
  pin_rst=PIN_RST_AT89S52;
  pin_vcc=PIN_VCC_AT89S52;
  pin_clock1=PIN_CLOCK1_AT89S52;
  pin_clock2=PIN_CLOCK2_AT89S52;
}

void edit_chip_model(void)
{
  int c;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT CHIP MODEL ***\n\n");
    print_chip_model();
    printf("\n");
    printf("A) AT98S52     B) *******\n");
    printf("C) *******     D) *******\n");
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': select_model_at89s52(); break;

      case 'Q': return; break;
    }
  }
}

/*
 *  disp_lock_bits()
 *
 */
void display_lock_bits(void)
{
  if(IS_AT89S52)
  {
    printf("Lock bits:\n\n");
    printf("LOCK BIT 1= %d\n",lock_bit_1 ? 1 : 0);
    printf("LOCK BIT 2= %d\n",lock_bit_2 ? 1 : 0);
    printf("LOCK BIT 3= %d\n",lock_bit_3 ? 1 : 0);
  }
}

/*
 *  edit_lock_bits()
 *
 *  edit lock bits
 *
 */
void edit_lock_bits(void)
{
  int c;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT LOCK BITS ***\n\n");
    display_lock_bits();
    printf("\n");
    if(IS_AT89S52)
    {
      printf("A) LOCK BIT1 <- 0     B) LOCK BIT1 <- 1\n");
      printf("C) LOCK BIT2 <- 0     D) LOCK BIT2 <- 1\n");
    }
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': lock_bit_1=0; break;
      case 'B': lock_bit_1=1; break;
      case 'C': lock_bit_2=0; break;
      case 'D': lock_bit_2=1; break;
      case 'Q': return; break;
    }
  }
}

/*
 *  read_lock_bits()
 *
 *  read lock bits
 *  from chip
 *
 */
void read_lock_bits(void)
{
  unsigned char fuse;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ LOCK BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();
  cmd_programming_enable();
  wait(10.0);
  fuse=cmd_read_lock_bits();
  all03_init_h();

  lock_bit_1=0;
  lock_bit_2=0;
  lock_bit_3=0;
  if(fuse & BIT(2)) lock_bit_1=1;
  if(fuse & BIT(3)) lock_bit_2=1;
  if(fuse & BIT(4)) lock_bit_3=1;

  display_lock_bits();
  printf("OK\n");
}

/*
 *  write_lock_bits()
 *
 */
void write_lock_bits(void)
{
  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE LOCK BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  chip_init();

  cmd_programming_enable();
  wait(10.0);
  cmd_write_lock_bits(lock_bit_1,lock_bit_2);

  all03_init_h();

  printf("OK\n");
}

/*
 *  chip_erase()
 *
 *  erase flash, eeprom and lock bits
 *  fuses unchanged
 *
 */
void chip_erase(void)
{
  clear_screen();
  if(test_if_timer()) return;
  printf("** ERASE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("ERASING...\n");
  chip_init();

  cmd_programming_enable();
  //wait(10.0);
  cmd_chip_erase();
  wait(100.0);

  all03_init_h();

  printf("OK\n");
}

/*
 *  read_signature_bytes()
 *
 *  reads signature bytes
 *
 *  xxxxxxx
 *  SIG 0 - 0x00 manifactured by Atmel
 *  SIG 1 - 0x93 8k flash
 *  SIG 2 - 0x01 indicates AT90S8515 if SIG 1=0x93
 *
 */
void read_signature_bytes(void)
{
  int i;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ SIGNATURE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();
  
  //wait_enter();
  //printf("press enter");
  cmd_programming_enable();
  wait(10.0);

  signature[0]=cmd_read_signature_bytes(0);
  signature[1]=cmd_read_signature_bytes(0x100);
  signature[2]=cmd_read_signature_bytes(0x200);

  all03_init_h();

  for(i=0;i<3;i++)
    printf("Signature[%d]=%02X\n", i, signature[i]);

  printf("OK\n");
}

/*
 *  write_program_memory()
 *
 *  write flash buffer to uC flash memory
 *
 */
void write_program_memory(void)
{
  unsigned short int address;
  unsigned char d;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  chip_init();

  cmd_programming_enable();
  wait(10.0);

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    if((d=flash[address])!=0xff)
    {
      cmd_write_program_memory(address,d);
      wait(10.0);
      //if(wait_rdy_program(address,d))
      //{
      //  printf("\nTimeout in writing %02X data at address %04X!\n",d,address);
      //  goto write_program_memory_q;
      //}
    }
  }
write_program_memory_q:

  all03_init_h();
  printf("\n");

  printf("OK\n");
}

/*
 *  read_program_memory()
 *
 *  reads uC flash memory into flash buffer
 *
 */
void read_program_memory(void)
{
  unsigned short int address;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();

  cmd_programming_enable();
  wait(10.0);

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    flash[address] = cmd_read_program_memory(address);
  }

  all03_init_h();
  printf("\n");

  printf("OK\n");
}

/*
 *  blak_check()
 *
 *  verifies if all uC flash memory cells=0xff
 *
 */
void blank_check_program_memory(void)
{
  int err;
  unsigned short int address;

  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** BLANK CHECK PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("CHECKING...\n");
  chip_init();

  cmd_programming_enable();
  wait(10.0);

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    //if(cmd_read_program_memory(address)!=0xff) err++;
    if(cmd_read_program_memory(address)!=0xff) err++;
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Flash not blank - %d (0x%04X) errors!\n", err, err);
  else printf("Flash is blank!\n");

  printf("OK\n");
}

/*
 *  verify_program_memory()
 *
 *  verifies uC flash memory with flash buffer
 *
 */
void verify_program_memory(void)
{
  int err;
  unsigned short int address;

  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** VERIFY PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("VERIFYING...\n");
  chip_init();

  cmd_programming_enable();
  wait(10.0);

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    if(flash[address]!=cmd_read_program_memory(address)) err++;
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Counted %d errors!\n",err);
  else printf("No errors!\n");
  
  printf("OK\n");
}

/*
 *  read_flash_buf_from_file()
 *
 *  reads data for flash buffer from disk
 *
 */
void read_program_memory_buffer_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO FLASH BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, max_flash_address, 0xff);
}

/*
 *  write_flash_buf_to_file()
 *
 *  writes data from flash buffer to disk
 *
 */
void write_program_memory_buffer_to_file(void)
{
  clear_screen();
  printf("*** WRITE FLASH BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, max_flash_address);
}

/* displays help screen */
void display_help(void)
{
    printf("use: at89s52 [<option>] [<file_name>]\n");
    printf("option:\n");
    printf(" -s       skip processor speed test\n");
    printf(" -h       display this screen\n");
    printf(" -bxxx    set all03 base address\n");
    printf(" -tat89s52  select AT89S52\n");

    exit(1);
}

/* main program */
int main(int argc, char *argv[])
{
  int c, ac, skip, tbase;

  all03_init_h();
  disable_ctrl_c_break();

  ac=argc-1;
  skip=0;
  while(ac)
  {
    if(!strcmp(argv[ac],"-s") || !strcmp(argv[ac],"-S")) skip=1;
    else if(!strcmp(argv[ac],"-h") || !strcmp(argv[ac],"-H"))
    {
      display_help();
      return(1);
    }
    else if(!strncmp(argv[ac],"-b",2) || !strncmp(argv[ac],"-B",2))
    {
      sscanf(&argv[ac][2],"%x",&tbase);
      all03_set_base_address(tbase);
    }
    else if(!strcmp(argv[ac],"-tat89s52") || !strcmp(argv[ac],"-TAT89S52")) select_model_at89s52();
    else strcpy(flash_file_name,argv[ac]);
    ac--;
  }  

  if(!skip)
  {
    printf("TESTING SPEED...\n");
    if(!test_timer()) printf("TIMER OK\n");
    else printf("\n\007* ALL03 Error - Timing not OK!\n");
  }

  if(all03_test_base_address())
  {
    printf("\n\007* ALL03 Error - Programmer not found at base address %04x!\n",all03_get_base_address());
    printf("Press <ENTER>\n");
    wait_enter();
  }

  for(;;)
  {
    clear_screen();
    printf("-----------------------------------------------------------------\n");
    printf("-- ALL03 - AT89S52 SERIAL LOW VOLTAGE PROGRAMMING MODULE --------\n");
    printf("----------------------------------by FaSt - Ver 1.0ALPHA---------\n");
    printf("\n");
    print_chip_model();
    printf("\n");
    printf("    Menu:\n");
    printf("\n");
    printf("2 - Read program file\n");
    printf("3 - Save program file\n");
    printf("6 - Edit base address\n");
    printf("B - Blank check\n");
    printf("R - Read\n");
    printf("P - Program\n");
    printf("L - Read lock bits\n");
    printf("I - Write lock bits\n");
    printf("S - Read signature\n");
    printf("E - Erase\n");
    printf("K - Edit lock bits\n");
    printf("V - Verify\n");
    printf("T - Select chip model\n");
    //printf("{ - Test\n");
    //printf("} - Dump ALL03 registers\n");
    printf("Q - quit\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<=126)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);
    
    switch(c)
    {
      case '2': read_program_memory_buffer_from_file(); break;
      case '3': write_program_memory_buffer_to_file(); break;
      case '6': all03_edit_base(); break;

      case 'B': blank_check_program_memory(); break;
      case 'R': read_program_memory(); break;
      case 'S': read_signature_bytes(); break;
      case 'L': read_lock_bits(); break;
      case 'P': write_program_memory(); break;
      case 'I': write_lock_bits(); break;
      case 'E': chip_erase(); break;
      case 'K': edit_lock_bits(); break;
      case 'V': verify_program_memory(); break;
      case 'T': edit_chip_model(); break;

      case '{': test(); break;
      case '}': all03_dump(); break;

      case 'Q':
        enable_ctrl_c_break();
        return(0);
      break;
    }
    printf("Press <ENTER>\n");
    wait_enter();
  }
  return(0);
}

