/*
 *
 *  ALL03
 *
 *  26.07.2002
 *
 *  by Fabio Sturman
 *
 *  fabio.sturman@tiscali.it
 *
 *  90s8515.c
 *
 *  main module for 90s8515
 *
 */

/* Copyright (c) 2002 Fabio Sturman (fabio.sturman@tiscali.it)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>
#include "../include/all03.h"
#include "../include/all03ft.h"
#include "../include/keys.h"
#include "../include/sleep.h"
#include "../include/binmask.h"
#include "90s8515.h"

/* all vars used only in level 2 */
unsigned short int flash[MAX_FLASH_ADDRESS_H*256];
unsigned char eeprom[MAX_EEPROM_ADDRESS_H*256];

int lock_bit_1=1;
int lock_bit_2=1;
int spien=1;
int fstrt=1;

unsigned char signature[3];

char flash_file_name[256]="flash90.bin";
char eeprom_file_name[256]="eeprom90.bin";


/*- AT90S8515 Level 1 --------------------------------------*/

/* write to portb of uC */
void wr_pb(unsigned char byt)
{
  int i;

  i=0;
  all03_wr_ttl_buf(i, byt);
  all03_wr(ALL03_TTL_ID+i, all03_rd_ttl_buf(i));
}

/* read portb of uC */
unsigned char rd_pb(void)
{
  int i;

  i=0;
  wr_pb(0xff);
  return(all03_rd(ALL03_TTL_ID+i));
}

/* load command into uC */
void ld_cmd(int bs, unsigned char cmd)
{
  wr_xa1(1);
  wr_xa0(0);
  wr_bs(bs);
  wr_pb(cmd);
  /* latch data */
  wr_xtal1(0);
  wr_xtal1(1);
  wr_xtal1(0);
}

/* load address (low if bs=0, high if bs=1) into uC */
void ld_addr(int bs, unsigned char addr)
{
  wr_xa1(0);
  wr_xa0(0);
  wr_bs(bs);
  wr_pb(addr);
  /* latch data */
  wr_xtal1(0);
  wr_xtal1(1);
  wr_xtal1(0);
}

/* load data into uC */
void ld_data(int bs, unsigned char dat)
{
  wr_xa1(0);
  wr_xa0(1);
  wr_bs(bs);
  wr_pb(dat);
  /* latch data */
  wr_xtal1(0);
  wr_xtal1(1);
  wr_xtal1(0);
}

/* read data from uC */
unsigned char rd_chip_data(void)
{
  unsigned char uc;
  
  wr_noe(0);
  uc=rd_pb();
  wr_noe(1);
  return(uc);
}

/* put chip in programming mode */
void chip_init(void)
{
  all03_set_vcc(5.0);
  all03_set_vhh(12.0);
  all03_set_gnd_20();
  all03_set_vcc_pin(40,1);
  wr_nreset(0);
  wr_bs(0);

  wait(0.1);

  all03_set_vhh_pin(NRESET,1);

  wr_nwr(1);
  wr_noe(1);
  wr_xtal1(0);
  wr_xa0(1);
  wr_xa1(1);
  wr_bs(1);
  wr_rdy(1);
  wr_pb(0xff);
}

/* negative pulse /wr */
void pulse_nwr(void)
{
  wr_nwr(0);
  wr_nwr(0);
  wr_nwr(0);
  wr_nwr(1);
}

/* wait while rdy=0  with timeout
   if timeout returns 1
   no timeout returns 0
*/
int wait_rdy(void)
{
  unsigned long t, dt;

  dt=get_one_ms()*5;
  for(t=0;t<dt;t++)
  {
    if(rd_rdy()) return(0);
  }
  return(1);
}

/*- AT90S8515 Level 2 --------------------------------------*/

/* LOCK BITS AND FUSES ----------------------------------------------*/

/*
 *  disp_lock_fuse()
 *
 *  display lock bits and fuses
 *
 */
void disp_lock_fuse(void)
{
  printf("Fuse bits:\n\n");
  printf("FSTRT=      %d\n",fstrt ? 1 : 0);
  printf("SPIEN=      %d\n",spien ? 1 : 0);
  printf("LOCK BIT 1= %d\n",lock_bit_1 ? 1 : 0);
  printf("LOCK BIT 2= %d\n",lock_bit_2 ? 1 : 0);
}

/*
 *  edit_lock_fuse()
 *
 *  edit lock bits and fuses
 *
 */
void edit_lock_fuse(void)
{
  int c;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT LOCK BITS AND FUSES ***\n\n");
    disp_lock_fuse();
    printf("\n");
    printf("A) FSTRT <-     0     B) FSTRT <-     1\n");
    printf("C) SPIEN <-     0     D) SPIEN <-     1\n");
    printf("E) LOCK BIT1 <- 0     F) LOCK BIT1 <- 1\n");
    printf("G) LOCK BIT2 <- 0     H) LOCK BIT2 <- 1\n");
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': fstrt=0; break;
      case 'B': fstrt=1; break;
      case 'C': spien=0; break;
      case 'D': spien=1; break;
      case 'E': lock_bit_1=0; break;
      case 'F': lock_bit_1=1; break;
      case 'G': lock_bit_2=0; break;
      case 'H': lock_bit_2=1; break;
      case 'Q': return; break;
    }
  }
}

/*
 *  read_lock_fuse()
 *
 *  read lock bits and fuses
 *  from chip
 *
 */
void read_lock_fuse(void)
{
  unsigned char fuse;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ FUSE AND LOCK BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_LOCK_FUSE_BITS);
  wr_bs(HIGH);
  fuse=rd_chip_data();
  
  all03_init_h();

  lock_bit_1=0;
  lock_bit_2=0;
  spien=0;
  fstrt=0;
  if(fuse & BIT(7)) lock_bit_1=1;
  if(fuse & BIT(6)) lock_bit_2=1;
  if(fuse & BIT(5)) spien=1;
  if(fuse & BIT(0)) fstrt=1;

  disp_lock_fuse();
  printf("OK\n");
}

/*
 *  write_fuse()
 *
 *  write fuses
 *
 */
void write_fuse(void)
{
  unsigned char fuse;
  
  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE FUSE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");

  fuse=0xff;
  if(!spien) fuse&=~BIT(5);
  if(!fstrt) fuse&=~BIT(0);

  chip_init();

  ld_cmd(LOW, COMMAND_WRITE_FUSE_BITS);
  ld_data(LOW, fuse);
  wr_nwr(0);
  wait(1.5);
  wr_nwr(1);
  
  all03_init_h();

  printf("OK\n");
}

/*
 *  write_lock()
 *
 *  write lock bits
 *
 */
void write_lock(void)
{
  unsigned char fuse;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE LOCK BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");

  fuse=0xff;
  if(!lock_bit_1) fuse&=~BIT(1);
  if(!lock_bit_2) fuse&=~BIT(2);
  //printf("Lock byte = %02x\n",fuse);
  chip_init();

  ld_cmd(LOW, COMMAND_WRITE_LOCK_BITS);
  ld_data(LOW, fuse); /* low data byte */
  pulse_nwr();
  //wr_nwr(0);
  //wait(0.1);
  //wr_nwr(1);

  wait(10.0);
  if(wait_rdy()) printf("\n\007* ALL03 Error - Timeout in writing lock bits!\n");

  all03_init_h();

  printf("OK\n");
}

/* ERASE -----------------------------------------------------------*/

/*
 *  erase()
 *
 *  erase flash, eeprom and lock bits
 *  fuses unchanged
 *
 */
void erase(void)
{
  clear_screen();
  if(test_if_timer()) return;
  printf("** ERASE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("ERASING...\n");
  chip_init();
  
  ld_cmd(LOW, COMMAND_CHIP_ERASE);
  wr_nwr(0);
  wait(10.0);
  wr_nwr(1);
  
  all03_init_h();

  printf("OK\n");
}

/*
 *  read_signature()
 *
 *  reads signature bytes
 *
 *  SIG 0 - 0x00 manifactured by Atmel
 *  SIG 1 - 0x93 8k flash
 *  SIG 2 - 0x01 indicates AT90S8515 if SIG 1=0x93
 *
 */
void read_signature(void)
{
  int i;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ SIGNATURE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();
  
  ld_cmd(LOW, COMMAND_READ_SIGNATURE);

  for(i=0; i<3; i++)
  {
    ld_addr(LOW, i);
    signature[i]=rd_chip_data();
  }

  all03_init_h();

  for(i=0;i<3;i++)
    printf("Signature[%d]=%02x\n", i, signature[i]);

  printf("OK\n");
}

/* FLASH -------------------------------------------------------------*/

/*
 *  write_flash()
 *
 *  write flash buffer to uC flash memory
 *
 */
void write_flash(void)
{
  unsigned short int addrl, addrh;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE FLASH ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  chip_init();

  ld_cmd(0,COMMAND_WRITE_FLASH);

  for(addrh=0; addrh<MAX_FLASH_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);         /* hi  addr byte */ 
    for(addrl=0;addrl<256;addrl++)
    {
      ld_addr(LOW, addrl);        /* low addr byte */
      printf("Address: %02x%02x\r",addrh,addrl);    
  
      if((flash[addrh*256+addrl] & 0xff)!=0xff)
      {
        ld_data(LOW, flash[addrh*256+addrl] & 0xff); /* low data byte */
        pulse_nwr();
        if(wait_rdy())
        {
          printf("\n\007* ALL03 Error - Timeout in writing flash at address %04x!\n",addrh*256+addrl);
          goto write_flash_q;
        }
      }

      if((flash[addrh*256+addrl] / 256)!=0xff)
      {
        ld_data(HIGH, flash[addrh*256+addrl] / 256); /* hi  data byte */
        pulse_nwr();
        if(wait_rdy())
        {
          printf("\n\007* ALL03 Error - Timeout in writing flash at address %04x!\n",addrh*256+addrl);
          goto write_flash_q;
        }
      }
    }
  }

write_flash_q:

  all03_init_h();
  printf("\n");

  printf("OK\n");
}

/*
 *  read_flash()
 *
 *  reads uC flash memory into flash buffer
 *
 */
void read_flash(void)
{
  unsigned short int addrl, addrh;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ FLASH ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_FLASH);
  for(addrh=0; addrh<MAX_FLASH_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);                    /* hi  addr byte */ 
    for(addrl=0;addrl<256;addrl++)
    {
      ld_addr(LOW, addrl);                  /* low addr byte */
      printf("Address: %02x%02x\r",addrh,addrl);    
  
      wr_bs(LOW);
      flash[addrh*256+addrl]=rd_chip_data();     /* low data byte */
      wr_bs(HIGH);
      flash[addrh*256+addrl]+=256*rd_chip_data(); /* hi data byte */
    }
  }

  all03_init_h();
  printf("\n");

  printf("OK\n");
}


/* EEPROM ------------------------------------------------*/

/*
 *  read_eeprom()
 *
 *  reads uC eeprom memory into eeprom buffer
 *
 */
void read_eeprom(void)
{

  unsigned short int addrl, addrh;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ EEPROM ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_EEPROM);
  for(addrh=0; addrh<MAX_EEPROM_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);               /* hi  addr byte */ 
    for(addrl=0;addrl<256;addrl++)
    {
      ld_addr(LOW, addrl);              /* low addr byte */
      wr_bs(LOW);
      printf("Address: %02x%02x\r",addrh,addrl);    
      eeprom[addrh*256+addrl]=rd_chip_data();
    }
  }

  all03_init_h();
  printf("\n");

  printf("OK\n");

}

/*
 *  blank_check_eeprom()
 *
 *  verifies if all eeproms cells =0xff
 *
 */
void blank_check_eeprom(void)
{

  int err;
  unsigned short int addrl, addrh;
  
  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** BLANK CHECK EEPROM ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("CHECKING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_EEPROM);
  for(addrh=0; addrh<MAX_EEPROM_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);                    /* hi  addr byte */ 
    for(addrl=0; addrl<256; addrl++)
    {
      ld_addr(LOW, addrl);                   /* low addr byte */
      wr_bs(LOW);
      printf("Address: %02x%02x\r",addrh,addrl);    
      if(rd_chip_data()!=0xff) err++;
    }
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Eeprom not blank! %d errors!\n", err);
  else printf("Eeprom is blank\n");

  printf("OK\n");

}

/*
 *  verify_eeprom()
 *
 *  verifies uC eeprom with eeprom buffer
 *
 */
void verify_eeprom(void)
{
  int err;
  unsigned short int addrl, addrh;
  
  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** VERIFY EEPROM ***\n\n");
  printf(PROC_STRING);
  wait_enter();

  printf("VERIFYING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_EEPROM);
  for(addrh=0; addrh<MAX_EEPROM_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);                    /* hi  addr byte */ 
    for(addrl=0; addrl<256; addrl++)
    {
      ld_addr(LOW, addrl);                  /* low addr byte */
      wr_bs(LOW);
      printf("Address: %02x%02x\r",addrh,addrl);    
      if(eeprom[addrh*256+addrl]!=rd_chip_data()) err++;
    }
  }

  all03_init_h();
  printf("\n");

  printf("Errors: %d\n",err);
  
  printf("OK\n");
}

/*
 *  write_eeprom()
 *
 *  writes eeprom buffer into uC eeprom memory
 *
 */
void write_eeprom(void)
{
  unsigned short int addrl, addrh;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE EEPROM ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_WRITE_EEPROM);
  for(addrh=0; addrh<MAX_EEPROM_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);         /* hi  addr byte */ 
    for(addrl=0; addrl<256; addrl++)
    {
      ld_addr(LOW, addrl);        /* low addr byte */
      printf("Address: %02x%02x\r",addrh,addrl);    
  
      if(eeprom[addrh*256+addrl]!=0xff)
      {
        ld_data(LOW, eeprom[addrh*256+addrl]); /* low data byte */
        pulse_nwr();
        if(wait_rdy())
        {
          printf("\n\007* ALL03 Error - Timeout in writing eeprom at address %04x!\n",addrh*256+addrl);
          goto write_eeprom_q;
        }
      }
    }
  }
  
write_eeprom_q:
  
  all03_init_h();
  printf("\n");

  printf("OK\n");
}

/*
 *  blak_check()
 *
 *  verifies if all uC flash memory cells=0xff
 *
 */
void blank_check(void)
{
  int err;
  unsigned short int addrl, addrh;

  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** BLANK CHECK FLASH ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("CHECKING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_FLASH);
  for(addrh=0; addrh<MAX_FLASH_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);                    /* hi  addr byte */ 
    for(addrl=0; addrl<256; addrl++)
    {
      ld_addr(LOW, addrl);                  /* low addr byte */
      printf("Address: %02x%02x\r",addrh,addrl);    
  
      wr_bs(LOW);
      if(rd_chip_data()!=0xff) err++;     /* low data byte */
      wr_bs(HIGH);
      if(rd_chip_data()!=0xff) err++;    /* hi data byte */
    }
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Flash not blank! %d errors!\n", err);
  else printf("Flash is blank\n");

  printf("OK\n");
}

/*
 *  verify_flash()
 *
 *  verifies uC flash memory with flash buffer
 *
 */
void verify_flash(void)
{
  int err;
  unsigned short int addrl, addrh;

  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** VERIFY FLASH ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("VERIFYING...\n");
  chip_init();

  ld_cmd(LOW, COMMAND_READ_FLASH);
  for(addrh=0; addrh<MAX_FLASH_ADDRESS_H; addrh++)
  {
    ld_addr(HIGH, addrh);                    /* hi  addr byte */ 
    for(addrl=0;addrl<=255;addrl++)
    {
      ld_addr(LOW, addrl);                  /* low addr byte */
      printf("Address: %02x%02x\r",addrh,addrl);    
      wr_bs(LOW);
      if((flash[addrh*256+addrl] & 0xff) != rd_chip_data()) err++;     /* low data byte */
      wr_bs(HIGH);
      if((flash[addrh*256+addrl]/256) != rd_chip_data()) err++; /* hi data byte */
    }
  }

  all03_init_h();
  printf("\n");

  printf("Errors: %d\n",err);
  
  printf("OK\n");
}

/*
 *  read_flash_buf_from_file()
 *
 *  reads data for flash buffer from disk
 *
 */
void read_flash_buf_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO FLASH BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, MAX_FLASH_ADDRESS_H*512, 0xff);
}


/*
 *  write_flash_buf_to_file()
 *
 *  writes data from flash buffer to disk
 *
 */
void write_flash_buf_to_file(void)
{
  clear_screen();
  printf("*** WRITE FLASH BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, MAX_FLASH_ADDRESS_H*512);
}

/*
 *  read_eeprom_buf_from_file()
 *
 *  reads data for eeprom buffer from disk
 *
 */
void read_eeprom_buf_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO EEPROM BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    eeprom_file_name, eeprom, MAX_EEPROM_ADDRESS_H*256, 0xff);
}

/*
 *  write_eeprom_buf_to_file()
 *
 *  writes data from eeprom buffer to disk
 *
 */
void write_eeprom_buf_to_file(void)
{
  clear_screen();
  printf("*** WRITE EEPROM BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    eeprom_file_name, eeprom, MAX_EEPROM_ADDRESS_H*256);
}

/* displays help screen */
void display_help(void)
{
    printf("use: 90s8515 [<option>] [<file_name>]\n");
    printf("option:\n");
    printf(" -s, -S        skip processor speed test\n");
    printf(" -h, -H        display this screen\n");
    printf(" -bxxx, -Bxxx  set all03 base address\n");
    exit(1);
}

/* main program */
int main(int argc, char *argv[])
{
  int c, ac, skip, tbase;

  all03_init_h();
  disable_ctrl_c_break();

  ac=argc-1;
  skip=0;
  while(ac)
  {
    if(!strcmp(argv[ac],"-s") || !strcmp(argv[ac],"-S")) skip=1;
    else if(!strcmp(argv[ac],"-h") || !strcmp(argv[ac],"-H"))
    {
      display_help();
      return(1);
    }
    else if(!strncmp(argv[ac],"-b",2) || !strncmp(argv[ac],"-B",2))
    {
      sscanf(&argv[ac][2],"%x",&tbase);
      all03_set_base_address(tbase);
    }
    else strcpy(flash_file_name,argv[ac]);
    ac--;
  }  

  if(!skip)
  {
    printf("TESTING SPEED...\n");
    if(!test_timer()) printf("TIMER OK\n");
    else printf("\n\007* ALL03 Error - Timing not OK!\n");
  }

  if(all03_test_base_address())
  {
    printf("\n\007* ALL03 Error - Programmer not found at base address %04x!\n",all03_get_base_address());
    printf("Press <ENTER>\n");
    wait_enter();
  }

  for(;;)
  {
    clear_screen();
    printf("------------------------------------------------\n");
    printf("-- ALL03 - AT90S8515 PROGRAMMING MODULE --------\n");
    printf("-----------------by FaSt - Ver 1.0BETA4---------\n");
    printf("\n");
    printf("    Menu:\n");
    printf("\n");
    printf("2 - Read flash file           4 - Read eeprom file\n");
    printf("3 - Save flash file           5 - Save eeprom file\n");
    printf("6 - Edit base address\n");
    printf("B - Blank check flash         Z - Blank check eeprom\n");
    printf("R - Read flash                O - Read eeprom\n");
    printf("P - Program flash             F - Program eeprom\n");
    printf("L - Read fuse & lock bits\n");
    printf("I - Write lock bits           X - Write fuse bits\n");
    printf("S - Read signature\n");
    printf("E - Erase\n");
    printf("K - Edit fuse and lock bits\n");
    printf("V - Verify flash              U - Verify eeprom\n");
    //printf("{ - Test\n");
    //printf("} - Dump ALL03 registers\n");
    printf("Q - quit\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<=126)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);
    
    switch(c)
    {
      case '2': read_flash_buf_from_file(); break;
      case '3': write_flash_buf_to_file(); break;
      case '4': read_eeprom_buf_from_file(); break;
      case '5': write_eeprom_buf_to_file(); break;
      case '6': all03_edit_base(); break;

      case 'B': blank_check(); break;
      case 'Z': blank_check_eeprom(); break;
      case 'R': read_flash(); break;
      case 'O': read_eeprom(); break;
      case 'S': read_signature(); break;
      case 'L': read_lock_fuse(); break;
      case 'P': write_flash(); break;
      case 'F': write_eeprom(); break;
      case 'I': write_lock(); break;
      case 'X': write_fuse(); break;
      case 'E': erase(); break;
      case 'K': edit_lock_fuse(); break;
      case 'V': verify_flash(); break;
      case 'U': verify_eeprom(); break;

      case '{': test(); break;
      case '}': all03_dump(); break;

      case 'Q':
        enable_ctrl_c_break();
        return(0);
      break;
    }
    
    printf("Press <ENTER>\n");
    wait_enter();
  }
  return(0);
}

